import 'package:flutter/material.dart';

import 'unlock_pattern.dart';

class UnlockPatternIndicator extends StatefulWidget {
  ///圆之间的间距
  final double roundSpace;

  ///圆之间的间距比例(以圆直径作为基准)，[roundSpace]设置时无效
  final double roundSpaceRatio;

  ///线宽度
  final double strokeWidth;

  ///默认颜色
  final Color defaultColor;

  ///选中颜色
  final Color selectedColor;

  final _UnlockPatternIndicatorState _state = _UnlockPatternIndicatorState();

  UnlockPatternIndicator(
      {this.roundSpace,
      this.roundSpaceRatio = 0.5,
      this.strokeWidth = 1,
      this.defaultColor = Colors.grey,
      this.selectedColor = Colors.blue});

  void setSelectPoint(List<int> selected) {
    _state.setSelectPoint(selected);
  }

  @override
  _UnlockPatternIndicatorState createState() {
    return _state;
  }
}

class _UnlockPatternIndicatorState extends State<UnlockPatternIndicator> {
  List<Round> _rounds = List<Round>(9);
  double _radius;

  @override
  void didChangeDependencies() {
    super.didChangeDependencies();
    WidgetsBinding.instance.addPostFrameCallback(_init);
  }

  @override
  void didUpdateWidget(StatefulWidget oldWidget) {
    super.didUpdateWidget(oldWidget);
    WidgetsBinding.instance.addPostFrameCallback(_init);
  }

  @override
  Widget build(BuildContext context) {
    return CustomPaint(
        size: Size.infinite,
        painter: LockPatternIndicatorPainter(_rounds, _radius,
            widget.strokeWidth, widget.defaultColor, widget.selectedColor));
  }

  void setSelectPoint(List<int> selected) {
    for (int i = 0; i < _rounds.length; i++) {
      _rounds[i].status = selected.contains(i)
          ? UnlockPatternStatus.success
          : UnlockPatternStatus.normal;
    }
  }

  _init(_) {
    if (context.size.width > context.size.height) {
      throw Exception("widget width must <= height");
    }
    var width = context.size.width;
    var roundSpace = widget.roundSpace;
    if (roundSpace != null) {
      _radius = (width - roundSpace * 2) / 3 / 2;
    } else {
      _radius = width / (3 + widget.roundSpaceRatio * 2) / 2;
      roundSpace = _radius * 2 * widget.roundSpaceRatio;
    }

    for (int i = 0; i < _rounds.length; i++) {
      var row = i ~/ 3;
      var column = i % 3;
      var dx = column * (_radius * 2 + roundSpace) + _radius;
      var dy = row * (_radius * 2 + roundSpace) + _radius;
      _rounds[i] = Round(dx, dy, UnlockPatternStatus.normal);
    }
    setState(() {});
  }
}

class LockPatternIndicatorPainter extends CustomPainter {
  List<Round> _rounds;
  double _radius;
  double _strokeWidth;
  Color _defaultColor;
  Color _selectedColor;

  LockPatternIndicatorPainter(this._rounds, this._radius, this._strokeWidth,
      this._defaultColor, this._selectedColor);

  @override
  void paint(Canvas canvas, Size size) {
    if (_radius == null) return;

    var paint = Paint();
    paint.strokeWidth = _strokeWidth;

    for (var round in _rounds) {
      switch (round.status) {
        case UnlockPatternStatus.normal:
          paint.style = PaintingStyle.fill;
          paint.color = _defaultColor;
          canvas.drawCircle(round.toOffset(), _radius, paint);
          break;
        case UnlockPatternStatus.success:
          paint.style = PaintingStyle.fill;
          paint.color = _selectedColor;
          canvas.drawCircle(round.toOffset(), _radius, paint);
          break;
        default:
          break;
      }
    }
  }

  @override
  bool shouldRepaint(CustomPainter oldDelegate) {
    return true;
  }
}
